@@ -28,35 +28,63 @@ class SeoReqRewriter(SimpleReqRewriter):
         (rgx('/sitemap.xml$'), dict(vid='sitemapxml')),
         ]
 
+class RobotsRule(AppObject):
+    """abstract base class for robots rules
+
+    A custom rule should be built by extending this class. For instance::
+
+      from cubes.seo.views import RobotsRule
+
+      class VcsfileRobotsRule(RobotsRule):
+          __regid__ = 'vcsfile'
+          STANZA = ['User-Agent: *',
+                    'Disallow: /versioncontent',
+                    'Disallow: /testexecution',
+                    'Disallow: /versionedfile',
+                    'Disallow: /deletedversioncontent']
+
+    To get more control, override the items() method.
+    """
+    __registry__ = 'robotstxt'
+    __abstract__ = True
+    STANZA = []
+
+    def stanza(self):
+        return u'\n'.join(self.STANZA)
+
+class CwRobotsRule(RobotsRule):
+    __regid__ = 'cubicweb.admin'
+    STANZA =  ["User-Agent: *",
+               "Disallow: /schema",
+               "Disallow: /cwetype",
+               "Disallow: /cwrtype",
+               "Disallow: /search",
+               "Disallow: /login",
+               "Disallow: /cwuser",
+               "Disallow: /cwgroup",
+               "Disallow: /cwsource",
+               "Disallow: /_registry",
+               "Disallow: /siteconfig",
+               "Disallow: /siteinfo",
+               "Disallow: /notfound",
+               "Disallow: /processinfo",
+               "Disallow: /changelog",
+               "Disallow: /add/",
+               ]
+
 class RobotsTxt(View):
     """A view that implements the http://robotstxt.org standard"""
-
     __regid__ = 'robotstxt'
     content_type = 'text/plain'
     templatable = False
     __select__ = none_rset()
 
     def call(self):
-        lines = ["Sitemap: %s" % self._cw.build_url('sitemap.xml'),
-                 "",
-                 "User-Agent: *",
-                 "Disallow: /schema",
-                 "Disallow: /cwetype",
-                 "Disallow: /cwrtype",
-                 "Disallow: /search",
-                 "Disallow: /login",
-                 "Disallow: /cwuser",
-                 "Disallow: /cwgroup",
-                 "Disallow: /cwsource",
-                 "Disallow: /_registry",
-                 "Disallow: /siteconfig",
-                 "Disallow: /siteinfo",
-                 "Disallow: /notfound",
-                 "Disallow: /processinfo",
-                 "Disallow: /changelog",
-                 "Disallow: /add/",
-                 ]
-        self.w(u'\n'.join(lines))
+        self.w(u'Sitemap: %s\n\n' % self._cw.build_url('sitemap.xml'))
+        if 'robotstxt' in self._cw.vreg: # XXX or make sure it exists by creating it in this cube...
+            for rule in self._cw.vreg['robotstxt'].possible_objects(self._cw):
+                self.w(rule.stanza())
+                self.w(u'\n')
 
 CHANGEFREQS = frozenset(('always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never'))
 
